package de.hftstuttgart.unifiedticketing.core;

import de.hftstuttgart.unifiedticketing.exceptions.AssertionException;

import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Builder class for ticket systems
 * @param <B> type of builder implementation
 * @param <TS> type of ticket system this builder creates
 */
public abstract class TicketSystemBuilder<B extends TicketSystemBuilder, TS extends TicketSystem>
{
    private static Logger logger = Logging.getLogger(TicketSystemBuilder.class.getName());

    public String baseUrl;

    /**
     * set's the base url of a system
     * @param url base url without protocol prefix
     * @return this builder
     */
    public B withBaseUrl(String url)
    {
        if (url.startsWith("http://") || url.startsWith("https://"))
        {
            logger.log(Level.SEVERE, "base url not allowed to start with protocol!");
            throw new AssertionException(String.format("url %s started with protocol which is not allowed", url));
        }
        baseUrl = url;
        logger.log(Level.FINEST, "set base url to " + baseUrl);
        return (B) this;
    }

    /**
     * creates a new ticket system instance from the previously set information
     * @return new ticket system instance
     */
    public abstract TS build();
}
